@@ -71,7 +71,7 @@ enum {
 /*
  * How many depth levels to log
  */
-#define FIO_IO_U_MAP_NR	8
+#define FIO_IO_U_MAP_NR	7
 #define FIO_IO_U_LAT_U_NR 10
 #define FIO_IO_U_LAT_M_NR 12
 
@@ -120,8 +120,8 @@ struct thread_stat {
 	unsigned int io_u_complete[FIO_IO_U_MAP_NR];
 	unsigned int io_u_lat_u[FIO_IO_U_LAT_U_NR];
 	unsigned int io_u_lat_m[FIO_IO_U_LAT_M_NR];
-	unsigned long total_io_u[2];
-	unsigned long short_io_u[2];
+	unsigned long total_io_u[3];
+	unsigned long short_io_u[3];
 	unsigned long total_submit;
 	unsigned long total_complete;
 
@@ -255,11 +255,16 @@ struct thread_options {
 	unsigned int gtod_offload;
 	enum fio_cs clocksource;
 	unsigned int no_stall;
+	unsigned int trim_percentage;
+	unsigned int trim_batch;
+	unsigned int trim_zero;
+	unsigned long long trim_backlog;
 
 	char *read_iolog_file;
 	char *write_iolog_file;
 	char *bw_log_file;
 	char *lat_log_file;
+	char *replay_redirect;
 
 	/*
 	 * Pre-run and post-run shell
@@ -346,12 +351,14 @@ struct thread_data {
 
 	char *sysfs_root;
 
-	unsigned long rand_seeds[6];
+	unsigned long rand_seeds[7];
 
 	os_random_state_t bsrange_state;
 	os_random_state_t verify_state;
+	os_random_state_t trim_state;
 
 	unsigned int verify_batch;
+	unsigned int trim_batch;
 
 	int shm_id;
 
@@ -435,6 +442,12 @@ struct thread_data {
 	 */
 	struct flist_head io_log_list;
 
+	/*
+	 * For tracking/handling discards
+	 */
+	struct flist_head trim_list;
+	unsigned long trim_entries;
+
 	/*
 	 * for fileservice, how often to switch to a new file
 	 */
@@ -555,6 +568,8 @@ extern void options_mem_dupe(struct thread_data *);
 extern void options_mem_free(struct thread_data *);
 extern void td_fill_rand_seeds(struct thread_data *);
 extern void add_job_opts(const char **);
+extern char *num2str(unsigned long, int, int, int);
+
 #define FIO_GETOPT_JOB		0x89988998
 #define FIO_NR_OPTIONS		(FIO_MAX_OPTS + 128)
 
@@ -640,49 +655,6 @@ static inline int fio_fill_issue_time(struct thread_data *td)
 	return 0;
 }
 
-/*
- * Cheesy number->string conversion, complete with carry rounding error.
- */
-static inline char *num2str(unsigned long num, int maxlen, int base, int pow2)
-{
-	char postfix[] = { ' ', 'K', 'M', 'G', 'P', 'E' };
-	unsigned int thousand;
-	char *buf;
-	int i;
-
-	if (pow2)
-		thousand = 1024;
-	else
-		thousand = 1000;
-
-	buf = malloc(128);
-
-	for (i = 0; base > 1; i++)
-		base /= thousand;
-
-	do {
-		int len, carry = 0;
-
-		len = sprintf(buf, "%'lu", num);
-		if (len <= maxlen) {
-			if (i >= 1) {
-				buf[len] = postfix[i];
-				buf[len + 1] = '\0';
-			}
-			return buf;
-		}
-
-		if ((num % thousand) >= (thousand / 2))
-			carry = 1;
-
-		num /= thousand;
-		num += carry;
-		i++;
-	} while (i <= 5);
-
-	return buf;
-}
-
 static inline int __should_check_rate(struct thread_data *td,
 				      enum fio_ddir ddir)
 {
