@@ -22,7 +22,7 @@
 
 #include "lib/getopt.h"
 
-static char fio_version_string[] = "fio 1.43.2";
+static char fio_version_string[] = "fio 1.44";
 
 #define FIO_RANDSEED		(0xb1899bedUL)
 
@@ -447,6 +447,7 @@ void td_fill_rand_seeds(struct thread_data *td)
 		os_random_seed(td->rand_seeds[3], &td->next_file_state);
 
 	os_random_seed(td->rand_seeds[5], &td->file_size_state);
+	os_random_seed(td->rand_seeds[6], &td->trim_state);
 
 	if (!td_random(td))
 		return;
@@ -560,7 +561,7 @@ static int add_job(struct thread_data *td, const char *jobname, int job_add_num)
 	td->ts.slat_stat[0].min_val = td->ts.slat_stat[1].min_val = ULONG_MAX;
 	td->ts.lat_stat[0].min_val = td->ts.lat_stat[1].min_val = ULONG_MAX;
 	td->ts.bw_stat[0].min_val = td->ts.bw_stat[1].min_val = ULONG_MAX;
-	td->ddir_seq_nr = td->o.ddir_seq_nr + 1;
+	td->ddir_seq_nr = td->o.ddir_seq_nr;
 
 	if ((td->o.stonewall || td->o.new_group) && prev_group_jobs) {
 		prev_group_jobs = 0;
