@@ -1,12 +1,15 @@
 #ifndef FIO_IOENGINE_H
 #define FIO_IOENGINE_H
 
-#define FIO_IOOPS_VERSION	10
+#define FIO_IOOPS_VERSION	12
 
 enum {
-	IO_U_F_FREE	= 1 << 0,
-	IO_U_F_FLIGHT	= 1 << 1,
-	IO_U_F_FREE_DEF	= 1 << 2,
+	IO_U_F_FREE		= 1 << 0,
+	IO_U_F_FLIGHT		= 1 << 1,
+	IO_U_F_FREE_DEF		= 1 << 2,
+	IO_U_F_IN_CUR_DEPTH	= 1 << 3,
+	IO_U_F_BUSY_OK		= 1 << 4,
+	IO_U_F_TRIMMED		= 1 << 5,
 };
 
 /*
@@ -28,6 +31,9 @@ struct io_u {
 #endif
 #ifdef FIO_HAVE_SOLARISAIO
 		aio_result_t resultp;
+#endif
+#ifdef FIO_HAVE_BINJECT
+		struct b_user_cmd buc;
 #endif
 		void *mmap_data;
 	};
@@ -48,6 +54,12 @@ struct io_u {
 	void *xfer_buf;
 	unsigned long xfer_buflen;
 
+	/*
+	 * Parameter related to pre-filled buffers and
+	 * their size to handle variable block sizes.
+	 */
+	unsigned long buf_filled_len;
+
 	unsigned int resid;
 	unsigned int error;
 
@@ -152,6 +164,9 @@ extern void io_u_fill_buffer(struct thread_data *td, struct io_u *, unsigned int
 void io_u_mark_complete(struct thread_data *, unsigned int);
 void io_u_mark_submit(struct thread_data *, unsigned int);
 
+int do_io_u_sync(struct thread_data *, struct io_u *);
+int do_io_u_trim(struct thread_data *, struct io_u *);
+
 #ifdef FIO_INC_DEBUG
 static inline void dprint_io_u(struct io_u *io_u, const char *p)
 {
